/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.constraint;

import java.util.Collection;

import org.fortiss.af3.exploration.model.ExplorationConstraint;
import org.opt4j.core.problem.Decoder;
import org.opt4j.core.problem.Evaluator;

import com.google.common.collect.HashMultimap;
import com.google.common.collect.Multimap;

/**
 * Base class that holds the constraints that are specific for some artifact produced by the DSE. It
 * shall be used by operators and decoders to ensure the construction of valid artifacts. Only
 * "simple" constraints or ones produced by {@link Decoder}s shall be added to this map. For complex
 * constraints, the use of {@link Evaluator}s is advised.
 * 
 * @author diewald
 */
public abstract class ConstrainedArtifactMap<T> implements IConstrained<T> {

	/** Associates artifacts with constraints that a producer or modifier has to consider. */
	private Multimap<T, ExplorationConstraint<?>> constrainedArtifacts = HashMultimap.create();

	/** {@inheritDoc} */
	@Override
	public Collection<ExplorationConstraint<?>> getConstraintsOf(T constrainedArtifact) {
		return constrainedArtifacts.get(constrainedArtifact);
	}

	/** {@inheritDoc} */
	@Override
	public void putConstraints(T constrainedArtifact,
			Collection<ExplorationConstraint<?>> constraints) {
		constrainedArtifacts.putAll(constrainedArtifact, constraints);
	}

	/** {@inheritDoc} */
	@Override
	public void removeArtifact(T constrainedArtifact) {
		constrainedArtifacts.removeAll(constrainedArtifact);
	}
}
