/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.constraint;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.Opt4JDseTaskProvider;
import org.opt4j.core.Genotype;
import org.opt4j.core.Individual;
import org.opt4j.core.IndividualSet;
import org.opt4j.core.IndividualSetListener;
import org.opt4j.core.genotype.CompositeGenotype;

import com.google.inject.Inject;
import com.google.inject.Injector;
import com.google.inject.Provider;
import com.google.inject.Singleton;

/**
 * Instance of the {@link ConstrainedArtifactMap} for {@link Genotype}s. Here, the artifacts are
 * {@link Genotype}s INSTANCES since the genotypes already exist when being processed by Operators
 * that modify them. This {@link ConstrainedArtifactMap} is intended to be used by Operators.
 * 
 * @author diewald
 */
@Singleton
public class ConstrainedGenotypeMap extends ConstrainedArtifactMap<Genotype> {

	/**
	 * Constructor.
	 * 
	 * @param injectorProvider
	 *            References the Opt4J {@link Injector} {@link Provider} to install the listeners.
	 */
	@Inject
	public ConstrainedGenotypeMap(Opt4JDseTaskProvider injectorProvider) {
		IndividualSet individuals = injectorProvider.getInjector().getInstance(IndividualSet.class);
		individuals.addListener(new IndividualSetListener() {

			@Override
			public void individualRemoved(IndividualSet collection, Individual individual) {
				Genotype genotype = individual.getGenotype();
				removeArtifact(genotype);
				if(genotype instanceof CompositeGenotype) {
					removeGenotypesRecursively((CompositeGenotype<?, ?>)genotype);
				}
			}

			@Override
			public void individualAdded(IndividualSet collection, Individual individual) {
				// TODO Add initially extracted constraints here.
			}
		});
	}

	/**
	 * Removes all associations in the backend map that are found in the given
	 * {@link CompositeGenotype}.
	 */
	private void removeGenotypesRecursively(CompositeGenotype<?, ?> compositeGenotype) {
		compositeGenotype.forEach(g -> {
			// FIXME: Assumption that we ALWAYS have the key as a class.
			removeArtifact((Genotype)g.getKey());
			if(g instanceof CompositeGenotype) {
				removeGenotypesRecursively((CompositeGenotype<?, ?>)g);
			}
		});
	}
}
