/*-------------------------------------------------------------------------+
| Copyright 2019 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype;

import java.util.ArrayList;
import java.util.Collection;

import org.opt4j.core.Genotype;
import org.opt4j.core.Individual;

import com.google.common.collect.HashMultimap;
import com.google.common.collect.Multimap;
import com.google.inject.AbstractModule;
import com.google.inject.Singleton;

/**
 * Holds the {@link Phenotype}s that can be reused in the offspring {@link Individual}s. These
 * {@link Phenotype}s have to be independent from {@link Genotype}s that have undergone a
 * modification.
 * 
 * @author diewald
 */
@Singleton
public class PreservablePhenotypeHolder extends AbstractModule {

	/** Holds preserved {@link Phenotype}s. */
	protected Multimap<Genotype, Phenotype> preserveMap = HashMultimap.create();

	/**
	 * Adds the given {@link Genotype} (key) and the collection of preservable {@link Phenotype}s to
	 * {@code this} holder.
	 */
	public void definePreservablePhenosOf(Genotype genotype,
			Collection<? extends Phenotype> preservePhenos) {
		preserveMap.putAll(genotype, preservePhenos);
	}

	/**
	 * Returns the collection of preserved {@link Phenotype}s for the given {@link Genotype} and
	 * removes its reference from the holder.
	 */
	public Collection<Phenotype> getClearPreservedPhenosFor(Genotype genotype) {
		Collection<Phenotype> preservedPhenos = new ArrayList<>(preserveMap.get(genotype));
		preserveMap.removeAll(genotype);
		return preservedPhenos;
	}
}
