/*-------------------------------------------------------------------------+
| Copyright 2015 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem.taskmapping.instantiate;

import java.util.HashMap;
import java.util.Map;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.abstractmapping.AbstractTaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.abstractmapping.AbstractTaskMappingEntry;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.instantiatedmapping.FailSilentTaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.instantiatedmapping.FailSilentTaskMappingEntry;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.decode.Decodes;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.decode.Genotyped;
import org.fortiss.af3.exploration.alg.dse.module.DecoderModule;
import org.opt4j.core.Genotype;

import com.google.inject.Provides;

/**
 * Decoder for instantiating an {@link AbstractTaskMappingEncoding} to a
 * {@link FailSilentTaskMappingEncoding}.
 * 
 * @author diewald
 */
// FIXME: The type hierarchy and naming of the classes is extremely confusing. FIX ASAP!
public class FailSilentAbstractTaskMappingDecoder
		extends DecoderModule<FailSilentTaskMappingEncoding> {

	// TODO: Abstract to the AbstractTaskMappingDecoder.
	// FIXME: Handle the case where no previous FailSilentTaskMappingEncoding exists.
	/**
	 * Decodes an {@link AbstractTaskMappingEncoding} into a {@link FailSilentTaskMappingEncoding}
	 * by instantiating abstract tasks. If a {@link FailSilentTaskMappingEncoding} exists from a
	 * previous iteration, this encoding is updated.
	 * 
	 * @param atmEnc
	 *            task mapping that contains potentially abstract tasks (no implementation).
	 * @param fstmEnc
	 *            task mapping that shall contain only tasks backed by an implementation.
	 * @return task mapping that contains only tasks backed by an implementation.
	 */
	@Provides
	@Decodes
	public FailSilentTaskMappingEncoding decode(AbstractTaskMappingEncoding atmEnc,
			@Genotyped FailSilentTaskMappingEncoding fstmEnc) {
		updateAssociations(atmEnc, fstmEnc);

		return fstmEnc;
		// for(ITaskAdapter<?> comp : atmEnc.getRequesters()) {
		// Collection<AbstractTaskMappingEntry> abstrEntries = atmEnc.getMappingEntriesOf(comp);
		// Collection<FailSilentTaskMappingEntry> instEntries;
		// if(comp.isInstantiationRequired()) {
		// instEntries = filterSet(fstmEnc.getMappingEntries(),
		// e -> e.getSource().getReplacedComponent() == comp);
		// }
		// // else {
		// // instEntries = fstmEnc.getMappingEntriesOf(comp);
		// // // Update assoc data
		// // for(FailSilentTaskMappingEntry entry : instEntries) {
		// //
		// // }
		// // }
		//
		// // Missing Associations between instantiated and abstract entries indicate the need to
		// // create or remove corresp. entries.
		// // for(AbstractTaskMappingEntry abstrEntry : abstrEntries) {
		// // // if(abstrEntry.getAssociatedElement().isEmpty()) {
		// // fstmEnc.createAndAddEntry(abstrEntry);
		// // // }
		// // }
		// //
		// // Collection<FailSilentTaskMappingEntry> rmEntries = new ArrayList<>();
		// // for(FailSilentTaskMappingEntry instEntry : instEntries) {
		// // if(instEntry.getAssociatedElement().isEmpty()) {
		// // rmEntries.add(instEntry);
		// // }
		// // }
		// // rmEntries.forEach(e -> fstmEnc.removeEntry(e));
		// }
		//
		// // Remove all tasks that are present in the output encoding but not in the input
		// encoding.
		// Collection<ITaskAdapter<?>> rmTasks = new ArrayList<>();
		// for(ITaskAdapter<?> comp : fstmEnc.getRequesters()) {
		// ITaskAdapter<?> rmComponent = comp;
		// if(comp.isInstantiation()) {
		// comp = comp.getReplacedComponent();
		// }
		// if(!atmEnc.getRequesters().contains(comp)) {
		// rmTasks.add(rmComponent);
		// }
		// }
		// rmTasks.forEach(e -> fstmEnc.removeTask(e));
		//
		// for(FailSilentTaskMappingEntry entry : fstmEnc.getMappingEntries()) {
		// entry.resetTarget(
		// entry.getAssociatedElement(AbstractTaskMappingEntry.class).getTarget());
		// }
		//
		// return fstmEnc;
	}

	/**
	 * Updates the associations between {@link AbstractTaskMappingEntry}s and
	 * {@link FailSilentTaskMappingEntry}s to account for newly constructed entries when
	 * {@link Genotype}s are copied.
	 * 
	 * @param atmEnc
	 *            Recent and decoded encoding of {@link AbstractTaskMappingEntry}s.
	 * @param fstmEnc
	 *            Recent encoding of {@link FailSilentTaskMappingEntry}.
	 */
	private void updateAssociations(AbstractTaskMappingEncoding atmEnc,
			FailSilentTaskMappingEncoding fstmEnc) {
		Map<FailSilentTaskMappingEntry, FailSilentTaskMappingEntry> oldNewEntryAssoc =
				new HashMap<>();
		for(FailSilentTaskMappingEntry fstmEntry : fstmEnc.getMappingEntries()) {
			AbstractTaskMappingEntry oldAssoc =
					fstmEntry.getAssociatedElement(AbstractTaskMappingEntry.class);
			FailSilentTaskMappingEntry oldFstmAssoc =
					oldAssoc.getAssociatedElement(FailSilentTaskMappingEntry.class);
			oldNewEntryAssoc.put(oldFstmAssoc, fstmEntry);
		}
		for(AbstractTaskMappingEntry atmEntry : atmEnc.getMappingEntries()) {
			FailSilentTaskMappingEntry oldFstmEntry =
					atmEntry.getAssociatedElement(FailSilentTaskMappingEntry.class);
			FailSilentTaskMappingEntry newFstmEntry = oldNewEntryAssoc.get(oldFstmEntry);
			newFstmEntry.resetTarget(atmEntry.getTarget());
			newFstmEntry.addAssociatedElement(atmEntry);
			atmEntry.addAssociatedElement(newFstmEntry);
		}
	}

	/** {@inheritDoc} */
	@Override
	public FailSilentTaskMappingEncoding decode(Genotype genotype) {
		// TODO Auto-generated method stub
		return null;
	}
}
