/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.evaluator.constraint.internal;

import static org.fortiss.tooling.common.util.LambdaUtils.getFirst;

import java.util.Collection;
import java.util.HashSet;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.partitionmapping.Partition;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.partitionmapping.PartitionMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.TaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.instantiatedmapping.FailSilentTaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.decode.Genotyped;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.evaluate.annotations.Evaluate;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.evaluate.annotations.Validates;
import org.fortiss.af3.exploration.alg.dse.evaluator.BooleanValue;
import org.fortiss.af3.exploration.alg.dse.evaluator.constraint.ConstraintEvaluatorBase;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IRequestAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IResourceAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITaskAdapter;
import org.fortiss.af3.exploration.moea.model.constraints.AssignmentConstraint;
import org.fortiss.af3.exploration.moea.model.constraints.InternalConstraint;
import org.opt4j.core.problem.Evaluator;

import com.google.inject.Provides;

/**
 * {@link Evaluator} for {@link AssignmentConstraint}s that are {@link InternalConstraint}s. Hence,
 * this {@link Evaluator} only provides a meaningful validate method.
 * 
 * @author diewald
 */
public class AssignmentConstraintEvaluator
		extends ConstraintEvaluatorBase<Boolean, AssignmentConstraint, TaskMappingEncoding<?, ?>> {

	/** Prototype Constructor. */
	public AssignmentConstraintEvaluator(Class<AssignmentConstraint> explorationTargetType) {
		super(explorationTargetType);
	}

	/** Constructor. */
	public AssignmentConstraintEvaluator(AssignmentConstraint explorationTarget) {
		super(explorationTarget);
	}

	/** Dummy evaluation. */
	@Provides
	@Evaluate(AssignmentConstraintEvaluator.class)
	public BooleanValue evaluateGoal() {
		return new BooleanValue(false);
	}

	/**
	 * Validates that {@link IRequestAdapter}s are only mapped to {@link IResourceAdapter}s that are
	 * allowed by the given constraint.
	 */
	@Provides
	@Validates
	// TODO(AD): Find a way in guice to retrieve all assignment encodings
	public Boolean validate(FailSilentTaskMappingEncoding tmEnc,
			@Genotyped PartitionMappingEncoding pmEnc) {
		// TODO(AD): The assignment constraints must be translated in the preprocessing to produce
		// multiple assignment constraints that match the enabled features to consider multi-layer
		// mappings.
		for(Object reqObj : explorationTarget.getSources()) {
			IRequestAdapter<?> reqAdp = (IRequestAdapter<?>)reqObj;
			Collection<IResourceAdapter<?>> targetAdp =
					(Collection<IResourceAdapter<?>>)(Collection<?>)tmEnc
							.getAllocatedResourcesFor((ITaskAdapter<?>)reqAdp);
			if(getFirst(targetAdp).orElse(null) instanceof Partition) {
				Collection<IResourceAdapter<?>> physTgts = new HashSet<>();
				targetAdp.forEach(
						p -> physTgts.addAll(pmEnc.getAllocatedResourcesFor((Partition)p)));
				targetAdp = physTgts;
			}
			if(!(explorationTarget.getTargets().containsAll(targetAdp))) {
				return false;
			}
		}
		return true;
	}
}
