/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.evaluator.objective;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.partitionmapping.PartitionMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.instantiatedmapping.FailSilentTaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.decode.Genotyped;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.evaluate.annotations.Evaluate;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.Phenotype;
import org.fortiss.af3.exploration.alg.dse.module.EvaluatorModule;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.mappers.TaskMapper;
import org.fortiss.af3.exploration.alg.exception.ExplorationRuntimeException;
import org.fortiss.af3.exploration.dsl_v2.model.expression.IExpression;
import org.fortiss.af3.exploration.model.ExplorationObjective;
import org.fortiss.af3.exploration.model.ExplorationTarget;
import org.fortiss.af3.exploration.smt.model.SMTObjective;
import org.opt4j.core.DoubleValue;

import com.google.inject.Provides;

/**
 * Evaluator for {@link IExpression}s of {@link ExplorationObjective}s. The actual evaluation
 * function resides in the evaluate method.
 * 
 * @author diewald
 */
public class MappingEvaluatorObjective
		extends EvaluatorModule<DoubleValue, ExplorationTarget<DoubleValue>, Phenotype> {

	/** Constructor. */
	public MappingEvaluatorObjective(
			Class<? extends ExplorationTarget<DoubleValue>> explorationTargetType) {
		super(explorationTargetType);
	}

	/** Constructor. */
	// TODO: The casting should not be needed: This is an implementation flaw to be resolved in the
	// corresponding "newInstance" implementations.
	public MappingEvaluatorObjective(SMTObjective explorationTarget) {
		super(explorationTarget);
	}

	/**
	 * For a given {@link ExplorationObjective}, this evaluation function executes the
	 * {@link MappingParserObjective} parser that evaluates the given {@link IExpression} subject to
	 * the current {@link FailSilentTaskMappingEncoding} and {@link PartitionMappingEncoding}.
	 */
	@Provides
	@Evaluate(MappingEvaluatorObjective.class)
	public DoubleValue evaluate(FailSilentTaskMappingEncoding mapping,
			@Genotyped PartitionMappingEncoding virtMapping, TaskMapper<EObject> taskMapper) {
		MappingParserObjective parser = new MappingParserObjective(mapping, virtMapping,
				explorationTarget.getExpression(), taskMapper);
		Number retVal = null;
		try {
			retVal = parser.parse();
		} catch(Exception e) {
			throw new ExplorationRuntimeException(
					"Failed to parse the expression " + explorationTarget.getName() + ".", e);
		}

		return new DoubleValue(retVal.doubleValue());
	}
}
