/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.module;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.evaluate.ICompositeEvaluator;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.Phenotype;
import org.fortiss.af3.exploration.model.ExplorationTarget;

import com.google.inject.Module;

/**
 * An {@link IEvaluatorModule} performs a quantification of {@link Phenotype}s (solutions). It is
 * activated and configured by user-specified {@link ExplorationTarget}s. {@link IEvaluatorModule}s
 * are specific to {@link ExplorationTarget} types.
 * 
 * @author diewald
 */
public interface IEvaluatorModule<R, T extends ExplorationTarget<R>, P extends Phenotype>
		extends Module, ICompositeEvaluator<R> {

	/**
	 * Returns the {@link ExplorationTarget} type for which {@code this} {@link EvaluatorModule} is
	 * applicable.
	 */
	Class<T> getExplorationTargetType();

	/** Returns the {@link ExplorationTarget} instance for which the evaluation is performed. */
	T getExplorationTarget();

	/** Returns the priority of the {@link IEvaluatorModule}. */
	int getPriority();

	/** Returns a new instance of {@code this} {@link IEvaluatorModule}. */
	<M extends IEvaluatorModule<R, T, P>> M newInstance(T explorationTarget);
}
