/*-------------------------------------------------------------------------+
| Copyright 2014 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.plot;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.solution.ResourceSchedule;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.solution.StrictTTSchedule;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.solution.TimeSlot;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IGatewayUnitAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IResourceAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITransmissionUnitAdapter;
import org.jfree.chart.labels.CategoryItemLabelGenerator;
import org.jfree.data.category.CategoryDataset;

/**
 * 
 * @author diewald
 */
public class ScheduleLabelGenerator implements CategoryItemLabelGenerator {

	/** holds row from previous call */
	private int rowLabel;
	/** holds column from previous call */
	private int colLabel;
	/** holds the time slot index for task naming */
	private int indexLabel;

	/** Refers to the schedule that is displayed. */
	private StrictTTSchedule<?, ?> schedule;

	/** Constructor. */
	public ScheduleLabelGenerator(StrictTTSchedule<?, ?> schedule) {
		this.schedule = schedule;
	}

	/** {@inheritDoc} */
	@Override
	public String generateLabel(CategoryDataset dataset, int row, int column) {
		// TODO: These variables are state trackers. Remove them. The index should be set
		// externally.
		// For the comments, see the method getItemPaint(...). This method is basically a
		// clone.
		if(rowLabel != row || colLabel != column) {
			rowLabel = row;
			colLabel = column;
			indexLabel = 0;
		}

		String retName = "";

		Comparable<?> rkey = dataset.getRowKey(row);
		Comparable<?> ckey = dataset.getColumnKey(column);

		if(rkey.equals("processing")) {
			/* Select the resource that corresponds to the given row */
			IResourceAdapter<?> resource = null;

			for(IResourceAdapter<?> currentResource : schedule.getResources()) {
				if(currentResource.getName().equals(ckey)) {
					resource = currentResource;
					break;
				}
			}

			/* Color tasks and voters differently */
			if(resource != null) {

				ResourceSchedule localSchedule = schedule.getResourceSchedule(resource);
				Object[] slots = localSchedule.keySet().toArray();

				if(indexLabel < slots.length) {
					TimeSlot slot = (TimeSlot)slots[indexLabel];
					// TODO hard coded color
					if(slot.isVoter()) {
						retName = "V";
					} else {
						if(resource instanceof ITransmissionUnitAdapter) {
							// TODO: In order to retrieve the source and target of the
							// transmission, it is first required to store the source and
							// the receiver of the message
							retName = "T";
						} else if(resource instanceof IGatewayUnitAdapter) {
							retName = "G";
						} else {
							retName = slot.getDeployedRequester().getSource().getName();
						}
					}
				}
			}
		}
		indexLabel++;
		return retName;
	}

	/** {@inheritDoc} */
	@Override
	public String generateRowLabel(CategoryDataset dataset, int row) {
		// TODO Auto-generated method stub
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public String generateColumnLabel(CategoryDataset dataset, int column) {
		// TODO Auto-generated method stub
		return null;
	}
}
