/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.service.module.base;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Map;

import org.fortiss.af3.exploration.alg.exception.ExplorationException;
import org.fortiss.af3.exploration.alg.service.module.IExpSpecTransformationModule;
import org.fortiss.af3.exploration.model.ExplorationTarget;

import com.google.inject.Injector;

/**
 * Base class for implementing simple converters for (parts of) AF3 elements into DDD-SE-internal
 * representations. Provides management code for the single {@link IExpSpecTransformationModule}s.
 * 
 * @author diewald
 */
public abstract class ExpSpecTransformationModuleBase
		implements IExpSpecTransformationModule<IExpSpecTransformationModule<?>> {

	/** Set of {@link IExpSpecTransformationModule}s registered in this module */
	protected Collection<IExpSpecTransformationModule<?>> modules = new ArrayList<>();

	/** {@inheritDoc} */
	@Override
	public Map<ExplorationTarget<?>, ExplorationTarget<?>> transform(Injector mainInjector,
			Map<ExplorationTarget<?>, ExplorationTarget<?>> inputExpTargets)
			throws ExplorationException {
		Map<ExplorationTarget<?>, ExplorationTarget<?>> transformedMap = inputExpTargets;
		for(IExpSpecTransformationModule<?> module : modules) {
			transformedMap.putAll(module.transform(mainInjector, inputExpTargets));
		}
		return transformedMap;
	}

	/** {@inheritDoc} */
	@Override
	public void registerModule(IExpSpecTransformationModule<?> module) {
		modules.add(module);
	}
}
